/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/inc/energylabelSettings
 *
 * @revision        $Revision: 1.2.32.2 $
 */
/*global define*/
/*jslint browser: true, regexp:true, unparam: true*/

define('de_epages/product/inc/energylabelsettings', [
    'jquery',
    "$tmpl!./energylabelsettings-scale",

    'jquery/ui/slider',
    'ep/ui/slider',
    '$ready!'
], function ($, tmplScale) {
    'use strict';

    return {
        init: function (opts) {
            var self = this,
                o;

            // extend default values
            self.opts = $.extend({
                labelsIsDefined: false,
                sourceIsDefined: false,
                numberOfLabels: 0,
                numberOfSystemLabels: 0,
                fromTo: '',
                labelMap: {},
                wrapper: $('body'),
                sourceInput: undefined
            }, opts);

            o = self.opts;
            o.fromTo = o.fromTo ? o.fromTo.split('-').map(function (str) {
                return str.trim();
            }) : [];

            // Add change event to select field where the merchant can change the highest efficiency class
            if (o.select) {
                self.$selectWrapper = o.select.closest('tr');
                o.select.on('change', $.proxy(self._select, self));
            }

            self.$wrapper = o.wrapper;
            self.singleSlider = self.$wrapper.find('#EngergyLabelSingleSlider');
            self.rangeSlider = self.$wrapper.find('#EngergyLabelRangeSlider');

            self.form = self.$wrapper.closest('form');
            // Observe the file input field
            o.sourceInput.on('change', function(evt) {
                self._change(evt.target.files);
            });

            o.sourceInput.on('delete', function (evt) {
                // Update values
                o.sourceIsDefined = false;
                o.labelsIsDefined = false;
                o.numberOfLabels = 0;


                // Energylabel file was deleted, so we delete also the label class
                self.$wrapper.find('#EnergyLabelStart').val('');
                self.$wrapper.find('#EnergyLabelEnd').val('');
                o.select.prop('disabled', true);

                o.sourceInput.trigger('change');
            });

            // Observe the radio buttons
            self.$wrapper.find('input[name=EnergyLabelSingleOrRange]').on('change', function (evt) {
                if (this.checked) {
                    if (Number(this.value) === 0) {
                        // Show single slider
                        self._toggleSlider(self.singleSlider, self.rangeSlider);
                    } else {
                        // Show range slider
                        self._toggleSlider(self.rangeSlider, self.singleSlider);
                    }
                }
            });

            // Initialize sliders
            self._initSlider(self.singleSlider, true);
            self._initSlider(self.rangeSlider, false);

            // wait for dimensions of slider
            window.setTimeout(function () {
                self._createScale(self.singleSlider);
            }, 150);

            // Setup
            self._change([]);
            self._select();

        },

        // Merchant changed the highest efficiency class
        _select: function () {
            this.highestClass = this.opts.select.val();
            this._updateLabelMap(this.highestClass);
            // Update slider handle
            this._checkSliderHandle(this.highestClass);
            this._updateGradient();
            this._updateArrow();
        },

        _checkSliderHandle: function (highestClass) {
            if (!this.activeSlider) {
                return;
            }

            var self = this,
                highestClass = highestClass || self.highestClass,
                highestClassIndex = self.opts.labelMap[highestClass],
                values = self.activeSlider.slider('values'),
                value = self.activeSlider.slider('value'),
                isRange = values.length,
                handle = isRange ? values[0] : value;

            if (handle < highestClassIndex) {
                // Update handle
                if (isRange) {
                    values[0] = highestClassIndex;
                    if (values[1] <= highestClassIndex) {
                        values[1] = highestClassIndex + 1;
                    }
                    self.activeSlider.slider('values', values);
                } else {
                    value = highestClassIndex;
                    self.activeSlider.slider('value', value);
                }

                // Set the hidden inputs correctly
                self.activeSlider.slider('option', 'stop').call(self.activeSlider, '', isRange ? {values: values} : {value: value});
            }
        },

        _updateLabelMap: function (highestClass) {
            var self = this,
                o = self.opts,
                labelMap = o.labelMap,
                isHigher = true,
                label,
                counter = 0;

            // Update the (CSS-)classes to get the correct color for the energy class
            for (label = 0; label < o.numberOfSystemLabels; label++) {
                labelMap[label].class = isHigher ? 'class-1' : 'class-' + (counter<6 ? counter+1 : 7);

                if (isHigher && labelMap[label].text === highestClass) {
                    isHigher = false;
                }

                if (!isHigher) {
                    counter++;
                }

            }
        },

        _updateGradient: function () {
            var self = this,
                o = self.opts,
                colorMap = {
                    'class-1': '#14661c',
                    'class-2': '#4db75a',
                    'class-3': '#bed556',
                    'class-4': '#fff04f',
                    'class-5': '#feb745',
                    'class-6': '#f46e37',
                    'class-7': '#ef192d'
                },
                width = 100/(o.numberOfSystemLabels-1),
                endOfFirstRange = width/2,
                gradient = 'linear-gradient(to right,  ',
                i, old, old2,
                highestClass = o.labelMap[o.numberOfSystemLabels - 1].class,
                breakpoint = endOfFirstRange;

            for (i = 0; i < o.numberOfSystemLabels; i++) {
                if (i === 0) {
                    gradient += colorMap[o.labelMap[i].class] + ' 0%,' + colorMap[o.labelMap[i].class] + ' ' + breakpoint + '%,';
                } else {
                    if (i === o.numberOfSystemLabels - 1) {
                        gradient += colorMap[o.labelMap[i].class] + ' ' + breakpoint + '%,' + colorMap[o.labelMap[i].class] + ' 100%)';
                    } else {
                        gradient += colorMap[o.labelMap[i].class] + ' ' + breakpoint + '%,' + colorMap[o.labelMap[i].class] + ' ' + (breakpoint + width) + '%,';
                        breakpoint += width;
                    }
                }
            };

            // Update gradient
            self.singleSlider.css({
                'background': gradient
            });
            self.rangeSlider.css({
                'background': gradient
            });

            // Add class for correct after
            old = self.singleSlider.attr('class').replace(/\sclass-./g, '');
            self.singleSlider.attr('class', old);
            self.singleSlider.addClass(highestClass);
            old2 = self.rangeSlider.attr('class').replace(/\sclass-./g, '');
            self.rangeSlider.attr('class', old2);
            self.rangeSlider.addClass(highestClass);
        },

        _change: function (files) {
            var self = this,
                o = self.opts,
                fileSelected = o.sourceIsDefined || !!files.length,
                isRange = ((o.labelsIsDefined && o.numberOfLabels > 1) || self.$wrapper.find('#RangeLabel')[0].checked) ? true : false;

            // Enable/Disable energylabel functionality
            self.$wrapper.find('input').prop('disabled', !fileSelected);
            self.singleSlider.slider('option', 'disabled', !fileSelected);
            self.rangeSlider.slider('option', 'disabled', !fileSelected);
            self.$wrapper.toggle(!!fileSelected);
            self.$selectWrapper.toggle(!!fileSelected);
            o.select.prop('disabled', !fileSelected);

            if (fileSelected) {
                if (isRange) {
                    self._toggleSlider(self.rangeSlider, self.singleSlider);
                } else {
                    self._toggleSlider(self.singleSlider, self.rangeSlider);
                }
            }
        },

        _createScale: function (singleSlider) {
            var self = this,
                o = self.opts,
                sliderWidth = singleSlider.width(),
                part = sliderWidth / (o.labelMap.labels.length - 1),
                tmpl = {
                    labels: o.labelMap.labels,
                    width: part,
                    half: part / 2,
                    length: o.labelMap.labels.length
                };

            // render template for scale
            tmplScale(tmpl).appendTo(o.wrapper.find('#EnergyLabelRangeWrapper'));
        },
        _initSlider: function (slider, single) {
            var self = this,
                o = self.opts,
                inputStart = o.wrapper.find('#EnergyLabelStart'),
                inputEnd = o.wrapper.find('#EnergyLabelEnd'),
                labelMap = o.labelMap,
                labelsIsDefined = o.labelsIsDefined,
                numberOfLabels = o.numberOfLabels,
                fromTo = o.fromTo,
                highestClass = o.select.val();

            slider.slider({
                disabled: true,
                min: 0,
                max: o.numberOfSystemLabels - 1,
                step: 1,
                range: single ? false : true,
                value: (single && labelsIsDefined && numberOfLabels === 1) ? labelMap[fromTo[0]] : labelMap[highestClass],
                values: !single ? ((labelsIsDefined && numberOfLabels > 1) ? [labelMap[fromTo[0]], labelMap[fromTo[1]]] : [labelMap[highestClass], o.numberOfSystemLabels - 1]) : null,
                start: function (event, ui) {
                    // trigger change to set the save button active if a form around the energy label wrapper exists
                    if (self.form.length) {
                        self.form.trigger('change');
                    }
                },
                slide: single
                    ? function (evt, ui) {
                        if (ui.value < o.labelMap[self.highestClass]) {
                            return false;
                        }

                        self._updateArrow(labelMap[ui.value].class, labelMap[ui.value].text);
                    }
                    : function (evt, ui) {
                        var label;

                        if (ui.values[0] < o.labelMap[self.highestClass]) {
                            return false;
                        }

                        if (ui.values[1] - ui.values[0] === 0) {
                            return false;
                        }

                        label = self._getMedian(ui.values[0], ui.values[1]);
                        self._updateArrow(labelMap[label].class, labelMap[ui.values[0]].text + ' - ' + labelMap[ui.values[1]].text);
                    },
                stop: single
                    ? function (evt, ui) {
                        // Disable second input
                        inputEnd.prop('disabled', true);

                        inputStart.val(labelMap[ui.value].text);
                        inputEnd.val('');

                    }
                    : function (evt, ui) {
                        // Enable second input
                        inputEnd.prop('disabled', false);

                        inputStart.val(labelMap[ui.values[0]].text);
                        inputEnd.val(labelMap[ui.values[1]].text);

                    }
            });
        },
        _updateArrow: function (labelClass, text) {
            if (!this.activeSlider) {
                return;
            }

            var self = this,
                o = self.opts,
                arrow = this.opts.wrapper.find('.energylabel-arrow'),
                old = arrow.attr('class').replace(/\sclass-./g, ''),
                values = self.activeSlider.slider('values'),
                value = self.activeSlider.slider('value'),
                isRange = values.length,
                position = isRange ? self._getMedian(values[0], values[1]) : value,
                text = text || (isRange ? o.labelMap[values[0]].text + ' - ' + o.labelMap[values[1]].text : o.labelMap[position].text);

            if (!labelClass) {
                labelClass = o.labelMap[position].class;
            }

            arrow.attr('class', old);
            arrow.addClass(labelClass);
            arrow.html(text);
        },

        // Attention: Returns position of the median inside the labelmap
        // This is not the energy class!
        // `start` and `end` are the positions of the slider handles which start with 0
        _getMedian: function (start, end) {
            var median = Math.ceil((end + start) / 2);
            return median;
        },
        _toggleSlider: function (enableSlider, disableSlider) {
            var self = this,
                values = enableSlider.slider('values'),
                value = enableSlider.slider('value'),
                isRange = values.length;

            self.activeSlider = enableSlider;

            enableSlider
                .show()
                .slider('enable');
            disableSlider
                .hide()
                .slider('disable');

            // Set the hidden inputs correctly
            enableSlider.slider('option', 'stop').call(enableSlider, '', isRange ? {values: values} : {value: value});
            // Check that no value is selected which lies below the highest efficiency class
            self._checkSliderHandle();
            self._updateArrow();
        }
    };
});