/*globals define*/
define([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/product/inc/energylabelsettings-test.html',
    'testsuite/sinon',
    'de_epages/product/inc/energylabelsettings',

    '$ready!'
], function ($, Q, fixture, sinon, energylabelsettings) {
    'use strict';

    var labelMap = {
            labels: ['A+++', 'A++', 'A+', 'A', 'B', 'C', 'D', 'E', 'F', 'G']
        },
        singleSlider,
        rangeSlider,
        sourceInput,
        $wrapper,
        initSliderSpy,
        toggleStub,
        inputStart,
        inputEnd;

        // Mock of the latest energy label classification
        labelMap[0] = {
          text: 'A+++',
          class: 'class-1'
        };
        labelMap['A+++'] = 0;


        labelMap[1] = {
          text: 'A++',
          class: 'class-2'
        };
        labelMap['A++'] = 1;


        labelMap[2] = {
          text: 'A+',
          class: 'class-3'
        };
        labelMap['A+'] = 2;


        labelMap[3] = {
          text: 'A',
          class: 'class-4'
        };
        labelMap['A'] = 3;


        labelMap[4] = {
          text: 'B',
          class: 'class-5'
        };
        labelMap['B'] = 4;


        labelMap[5] = {
          text: 'C',
          class: 'class-6'
        };
        labelMap['C'] = 5;


        labelMap[6] = {
          text: 'D',
          class: 'class-7'
        };
        labelMap['D'] = 6;


        labelMap[7] = {
          text: 'E',
          class: 'class-7'
        };
        labelMap['E'] = 7;


        labelMap[8] = {
          text: 'F',
          class: 'class-7'
        };
        labelMap['F'] = 8;


        labelMap[9] = {
          text: 'G',
          class: 'class-7'
        };
        labelMap['G'] = 9;

    QUnit.module('de_epages/product/inc/energylabelsettings', {
        setup: function () {
            fixture.init();
            initSliderSpy = sinon.spy(energylabelsettings, '_initSlider');
            toggleStub = sinon.stub(energylabelsettings, '_toggleSlider');
            singleSlider = $('#EngergyLabelSingleSlider');
            rangeSlider = $('#EngergyLabelRangeSlider');
            sourceInput = $('input[name=EnergyLabelSourceFile]');
            inputStart = $('#EnergyLabelStart');
            inputEnd = $('#EnergyLabelEnd');
            $wrapper = $('#EnergyLabelRangeWrapper');

            energylabelsettings.init({
                numberOfSystemLabels: 10,
                fromTo:  'B - G',
                labelMap: labelMap,
                wrapper: $('#EnergyLabelRangeWrapper'),
                sourceInput: sourceInput,
                select: $('#EnergyLabelHighestEfficiencyClass')
            });
        },
        teardown: function () {
            fixture.destroy();
            initSliderSpy.restore();
            toggleStub.restore();
        }
    });

    Q.test('After initialization', function () {
        Q.ok(
            energylabelsettings.opts.fromTo instanceof Array,
            '`fromTo`-String is used to generate an array'
        );

        Q.ok(
            initSliderSpy.calledTwice,
            'Both sliders are initialized'
        );

        Q.deepEqual(
            energylabelsettings.opts.fromTo,
            ['B', 'G'],
            'Energy classes are correctly set'
        );

        Q.ok(
            $wrapper.find('input').prop('disabled'),
            'Inputs are still disabled'
        );

        Q.ok(
            singleSlider.slider('option', 'disabled') && rangeSlider.slider('option', 'disabled'),
            'Sliders are still disabled'
        );

        Q.ok(
            !$wrapper.find('.energylabel-arrow').is(':visible'),
            'Arrow is still hidden'
        );

    });

    Q.test('_updateArrow', function () {
        var arrow = $('.energylabel-arrow');

        energylabelsettings.activeSlider = singleSlider;
        energylabelsettings._updateArrow('class-1', 'A+++++');
        Q.ok(
            arrow.hasClass('class-1'),
            'Correct class is set'
        );

        Q.strictEqual(
            arrow.html(),
            'A+++++',
            'Correct text is set'
        );

        energylabelsettings._updateArrow('class-3', 'B-----');
        Q.ok(
            arrow.hasClass('class-3'),
            'Correct class is set'
        );

        Q.strictEqual(
            arrow.html(),
            'B-----',
            'Correct text is set'
        );

        Q.equal(
            arrow.hasClass('class-1'),
            false,
            'Old label class is removed'
        );

        energylabelsettings.activeSlider = undefined;
    });

    Q.test('_getMedian', function () {
        Q.strictEqual(
            energylabelsettings._getMedian(0, 5),
            3,
            'Correct median is returned (average + worse energy label)'
        );

        Q.strictEqual(
            energylabelsettings._getMedian(2, 8),
            5,
            'Correct median is returned (average)'
        );
    });

    Q.test('_toggleSlider', function () {
        var updateArrowSpy = sinon.spy(energylabelsettings, '_updateArrow');

        // Disable stubbing
        toggleStub.restore();

        energylabelsettings.opts.wrapper.show();

        // Set to class 'E'
        singleSlider.slider('value', 7);
        // Set to class range 'A++ - D'
        rangeSlider.slider('values', [1, 6]);

        energylabelsettings._toggleSlider(singleSlider, rangeSlider);

        Q.strictEqual(
            inputStart.val(),
            'E',
            'Hidden input for start is set correctly'
        );

        Q.strictEqual(
            inputEnd.val(),
            '',
            'Other input for end is set correctly'
        );

        Q.ok(
            inputEnd.prop('disabled'),
            'Hidden input for end is disabled'
        );

        Q.ok(
            updateArrowSpy.called,
            'Arrow is updated'
        );

        Q.ok(
            singleSlider.is(':visible'),
            'Correct slider is visible'
        );

        Q.ok(
            !singleSlider.slider('option', 'disabled'),
            'Correct slider is enabled'
        );

         Q.ok(
            rangeSlider.slider('option', 'disabled'),
            'Other slider is disabled'
        );

        Q.ok(
            !rangeSlider.is(':visible'),
            'Other slider is hidden'
        );

        energylabelsettings._toggleSlider(rangeSlider, singleSlider);

        Q.strictEqual(
            inputStart.val(),
            'A++',
            'Hidden input for start is set correctly'
        );

        Q.strictEqual(
            inputEnd.val(),
            'D',
            'Hidden input for end is set correctly'
        );

        Q.ok(
            !inputEnd.prop('disabled'),
            'Hidden input for end is enabled'
        );

        Q.ok(
            rangeSlider.is(':visible'),
            'Correct slider is visible'
        );

        Q.ok(
            !singleSlider.is(':visible'),
            'Other slider is hidden'
        );
        Q.ok(
            singleSlider.slider('option', 'disabled'),
            'Correct slider is enabled'
        );

         Q.ok(
            !rangeSlider.slider('option', 'disabled'),
            'Other slider is disabled'
        );

        updateArrowSpy.restore();

        energylabelsettings.activeSlider = undefined;
    });

    Q.test('_initSlider', function () {
        energylabelsettings._initSlider(singleSlider, true);

        Q.ok(
            !singleSlider.slider('option', 'range'),
            'Single slider has no range'
        );

        energylabelsettings._initSlider(rangeSlider, false);

        Q.ok(
            rangeSlider.slider('option', 'range'),
            'Range slider has range'
        );
    });

    Q.asyncTest('Trigger file input changed or was deleted', function () {
        // Simulate there is already a file
        energylabelsettings.opts.sourceIsDefined = true;
        sourceInput.trigger('change');

        setTimeout(function () {
            Q.ok(
                !$wrapper.find('input').prop('disabled'),
                'Inputs are enabled'
            );

            Q.ok(
                !rangeSlider.slider('option', 'disabled') && !singleSlider.slider('option', 'disabled'),
                'Sliders are enabled'
            );

            Q.ok(
                toggleStub.called,
                '`_toggleSlider` was called'
            );

            Q.ok(
                $wrapper.find('.energylabel-arrow').is(':visible'),
                'Arrow is visible'
            );

            // 'Trigger file delete'
            sourceInput.trigger('delete');

            setTimeout(function () {
                Q.strictEqual(
                    inputStart.val(),
                    '',
                    'Hidden input for start is set correctly'
                );

                Q.strictEqual(
                    inputEnd.val(),
                    '',
                    'Hidden input for end is set correctly'
                );

                Q.ok(
                    $wrapper.find('input').prop('disabled'),
                    'Inputs are disabled'
                );

                Q.ok(
                    rangeSlider.slider('option', 'disabled') && singleSlider.slider('option', 'disabled'),
                    'Sliders are disabled'
                );

                Q.ok(
                    !$wrapper.find('.energylabel-arrow').is(':visible'),
                    'Arrow is hidden'
                );

                Q.start();
            }, 300);
        }, 500);
    });

    Q.asyncTest('Trigger changed radio button', function () {
        $wrapper.find('input[name=EnergyLabelSingleOrRange]')[0].checked = true;
        $($wrapper.find('input[name=EnergyLabelSingleOrRange]').get(0)).trigger('change');

        setTimeout(function () {
            Q.ok(
                toggleStub.called,
                '`_toggleSlider` is called'
            );

            Q.start();
        }, 300);
    });

});